@@ -64,11 +64,14 @@ enum
   SHEET_IS_FROZEN       = 1 << 1,                                     
   SHEET_IN_XDRAG        = 1 << 2,                                        
   SHEET_IN_YDRAG        = 1 << 3,                                        
-  SHEET_IN_SELECTION    = 1 << 4,
-  SHEET_ROW_FROZEN      = 1 << 5,  /* set rows to be resizeable */
-  SHEET_COLUMN_FROZEN   = 1 << 6,  /* set cols to be resizeable */
-  SHEET_AUTORESIZE      = 1 << 7,  /* resize column if text width is great than column width */
-  SHEET_ADJUST_TEXT     = 1 << 8,   /* fix text to column width */
+  SHEET_IN_DRAG         = 1 << 4,                                        
+  SHEET_IN_SELECTION    = 1 << 5,
+  SHEET_IN_RESIZE       = 1 << 6,
+  SHEET_IN_CLIP         = 1 << 7,
+  SHEET_ROW_FROZEN      = 1 << 8,  /* set rows to be resizeable */
+  SHEET_COLUMN_FROZEN   = 1 << 9,  /* set cols to be resizeable */
+  SHEET_AUTORESIZE      = 1 << 10,  /* resize column if text width is great than column width */
+  SHEET_ADJUST_TEXT     = 1 << 11, /* fix text to column width */
 }; 
 
 #define GTK_SHEET(obj)          GTK_CHECK_CAST (obj, gtk_sheet_get_type (), GtkSheet)
@@ -83,7 +86,10 @@ enum
 #define GTK_SHEET_IS_FROZEN(sheet)   (GTK_SHEET_FLAGS (sheet) & SHEET_IS_FROZEN)
 #define GTK_SHEET_IN_XDRAG(sheet)     (GTK_SHEET_FLAGS (sheet) & SHEET_IN_XDRAG)
 #define GTK_SHEET_IN_YDRAG(sheet)     (GTK_SHEET_FLAGS (sheet) & SHEET_IN_YDRAG)
+#define GTK_SHEET_IN_DRAG(sheet)     (GTK_SHEET_FLAGS (sheet) & SHEET_IN_DRAG)
 #define GTK_SHEET_IN_SELECTION(sheet) (GTK_SHEET_FLAGS (sheet) & SHEET_IN_SELECTION)
+#define GTK_SHEET_IN_RESIZE(sheet) (GTK_SHEET_FLAGS (sheet) & SHEET_IN_RESIZE)
+#define GTK_SHEET_IN_CLIP(sheet) (GTK_SHEET_FLAGS (sheet) & SHEET_IN_CLIP)
 #define GTK_SHEET_ROW_FROZEN(sheet)   (GTK_SHEET_FLAGS (sheet) & SHEET_ROW_FROZEN)
 #define GTK_SHEET_COLUMN_FROZEN(sheet)   (GTK_SHEET_FLAGS (sheet) & SHEET_COLUMN_FROZEN)
 #define GTK_SHEET_AUTORESIZE(sheet)   (GTK_SHEET_FLAGS (sheet) & SHEET_AUTORESIZE)
@@ -228,6 +234,9 @@ struct _GtkSheet{
 
   /* timer for automatic scroll during selection */  
   gint32 timer;
+  /* timer for flashing clipped range */
+  gint32 clip_timer;
+  gint interval;
 
   /* global selection button */
   GtkWidget *button;
@@ -290,9 +299,13 @@ struct _GtkSheet{
   /* the current y-pixel location of the xor-drag hline */
   gint y_drag;
 
-  /* current cell being dragging */
+  /* current cell being dragged */
   GtkSheetCell drag_cell;
+  /* current range being dragged */
+  GtkSheetRange drag_range;
 
+  /* clipped range */
+  GtkSheetRange clip_range;
 };
 
 struct _GtkSheetClass
@@ -308,6 +321,17 @@ struct _GtkSheetClass
  void (*select_range) (GtkSheet *sheet,
 	               GtkSheetRange *range);
 
+ void (*clip_range) (GtkSheet *sheet,
+	               GtkSheetRange *clip_range);
+
+ void (*resize_range)  (GtkSheet *sheet,
+	                 GtkSheetRange *old_range,
+                         GtkSheetRange *new_range);
+
+ void (*move_range)    (GtkSheet *sheet,
+	                 GtkSheetRange *old_range,
+                         GtkSheetRange *new_range);
+
  void (*activate_cell) (GtkSheet *sheet,
 	               gint row, gint column);
 
@@ -424,6 +448,13 @@ void
 gtk_sheet_select_column (GtkSheet * sheet,
 		         gint column);
 
+/* save selected range to "clipboard" */
+void
+gtk_sheet_clip_range (GtkSheet *sheet, GtkSheetRange range);
+/* free clipboard */
+void
+gtk_sheet_unclip_range(GtkSheet *sheet);
+
 /* get scrollbars adjustment */
 GtkAdjustment *
 gtk_sheet_get_vadjustment (GtkSheet * sheet);
